@@ -1,18 +1,17 @@
-use crate::RingBuffer;
+use crate::{RingBuffer, RingBufferExt, RingBufferRead, RingBufferWrite};
 use core::iter::FromIterator;
 use core::mem::MaybeUninit;
 use core::ops::{Index, IndexMut};
 
-/// The ConstGenericRingBuffer struct is a RingBuffer implementation which does not require `alloc`.
-/// However, it does require the still unstable rust feature `const-generics`. Therefore this struct
-/// is feature-gated behind the `const_generics` feature and when enabled only works on nightly rust.
+/// The `ConstGenericRingBuffer` struct is a `RingBuffer` implementation which does not require `alloc` but
+/// uses const generics instead.
 ///
 /// [`ConstGenericRingBuffer`] allocates the ringbuffer on the stack, and the size must be known at
 /// compile time through const-generics.
 ///
 /// # Example
 /// ```
-/// use ringbuffer::{ConstGenericRingBuffer, RingBuffer};
+/// use ringbuffer::{ConstGenericRingBuffer, RingBuffer, RingBufferExt, RingBufferWrite};
 ///
 /// let mut buffer = ConstGenericRingBuffer::<_, 2>::new();
 ///
@@ -39,27 +38,36 @@ pub struct ConstGenericRingBuffer<T, const CAP: usize> {
     writeptr: usize,
 }
 
+impl<T: Clone, const CAP: usize> Clone for ConstGenericRingBuffer<T, CAP> {
+    fn clone(&self) -> Self {
+        let mut new = ConstGenericRingBuffer::<T, CAP>::new();
+        for elem in self.iter() {
+            new.push(elem.clone())
+        }
+        new
+    }
+}
+
 // We need to manually implement PartialEq because MaybeUninit isn't PartialEq
-impl<T: 'static + PartialEq, const CAP: usize> PartialEq for ConstGenericRingBuffer<T, CAP> {
+impl<T: PartialEq, const CAP: usize> PartialEq for ConstGenericRingBuffer<T, CAP> {
     fn eq(&self, other: &Self) -> bool {
-        if self.len() != other.len() {
-            false
-        } else {
+        if self.len() == other.len() {
             for (a, b) in self.iter().zip(other.iter()) {
                 if a != b {
                     return false;
                 }
             }
             true
+        } else {
+            false
         }
     }
 }
 
-impl<T: 'static + PartialEq, const CAP: usize> Eq for ConstGenericRingBuffer<T, CAP> {}
+impl<T: PartialEq, const CAP: usize> Eq for ConstGenericRingBuffer<T, CAP> {}
 
 impl<T, const CAP: usize> ConstGenericRingBuffer<T, CAP> {
-    /// Creates a new RingBuffer. The method is here for compatibility with the alloc version of
-    /// RingBuffer. This method simply creates a default ringbuffer. The capacity is given as a
+    /// Creates a new `RingBuffer`. This method simply creates a default ringbuffer. The capacity is given as a
     /// type parameter.
     #[inline]
     pub fn new() -> Self {
@@ -85,35 +93,13 @@ impl<T, const CAP: usize> ConstGenericRingBuffer<T, CAP> {
     }
 }
 
-impl<T: 'static, const CAP: usize> RingBuffer<T> for ConstGenericRingBuffer<T, CAP> {
-    #[inline]
-    #[cfg(not(tarpaulin_include))]
-    fn capacity(&self) -> usize {
-        CAP
-    }
-
-    #[inline]
-    fn push(&mut self, value: T) {
-        if self.is_full() {
-            let index = crate::mask(self, self.readptr);
-            unsafe {
-                // make sure we drop whatever is being overwritten
-                // SAFETY: the buffer is full, so this must be inited
-                //       : also, index has been masked
-                // make sure we drop because it won't happen automatically
-                core::ptr::drop_in_place(self.buf[index].as_mut_ptr());
-            }
-            self.readptr += 1;
-        }
-        let index = crate::mask(self, self.writeptr);
-        self.buf[index] = MaybeUninit::new(value);
-        self.writeptr += 1;
-    }
-
+impl<T, const CAP: usize> RingBufferRead<T> for ConstGenericRingBuffer<T, CAP> {
     #[inline]
     fn dequeue_ref(&mut self) -> Option<&T> {
-        if !self.is_empty() {
-            let index = crate::mask(self, self.readptr);
+        if self.is_empty() {
+            None
+        } else {
+            let index = crate::mask(CAP, self.readptr);
             self.readptr += 1;
             let res = unsafe {
                 // SAFETY: index has been masked
@@ -121,12 +107,34 @@ impl<T: 'static, const CAP: usize> RingBuffer<T> for ConstGenericRingBuffer<T, C
             };
 
             Some(res)
-        } else {
-            None
         }
     }
 
-    impl_ringbuffer!(
+    impl_ringbuffer_read!(readptr);
+}
+
+impl<T, const CAP: usize> RingBufferWrite<T> for ConstGenericRingBuffer<T, CAP> {
+    #[inline]
+    fn push(&mut self, value: T) {
+        if self.is_full() {
+            let index = crate::mask(CAP, self.readptr);
+            unsafe {
+                // make sure we drop whatever is being overwritten
+                // SAFETY: the buffer is full, so this must be initialized
+                //       : also, index has been masked
+                // make sure we drop because it won't happen automatically
+                core::ptr::drop_in_place(self.buf[index].as_mut_ptr());
+            }
+            self.readptr += 1;
+        }
+        let index = crate::mask(CAP, self.writeptr);
+        self.buf[index] = MaybeUninit::new(value);
+        self.writeptr += 1;
+    }
+}
+
+impl<T, const CAP: usize> RingBufferExt<T> for ConstGenericRingBuffer<T, CAP> {
+    impl_ringbuffer_ext!(
         get_unchecked,
         get_unchecked_mut,
         readptr,
@@ -135,8 +143,20 @@ impl<T: 'static, const CAP: usize> RingBuffer<T> for ConstGenericRingBuffer<T, C
     );
 }
 
+impl<T, const CAP: usize> RingBuffer<T> for ConstGenericRingBuffer<T, CAP> {
+    #[inline]
+    #[cfg(not(tarpaulin_include))]
+    fn capacity(&self) -> usize {
+        CAP
+    }
+
+    impl_ringbuffer!(readptr, writeptr);
+}
+
 impl<T, const CAP: usize> Default for ConstGenericRingBuffer<T, CAP> {
     /// Creates a buffer with a capacity specified through the Cap type parameter.
+    /// # Panics
+    /// Panics if `CAP` is 0 or not a power of two
     #[inline]
     fn default() -> Self {
         assert_ne!(CAP, 0, "Capacity must be greater than 0");
@@ -152,7 +172,7 @@ impl<T, const CAP: usize> Default for ConstGenericRingBuffer<T, CAP> {
     }
 }
 
-impl<RB: 'static, const CAP: usize> FromIterator<RB> for ConstGenericRingBuffer<RB, CAP> {
+impl<RB, const CAP: usize> FromIterator<RB> for ConstGenericRingBuffer<RB, CAP> {
     fn from_iter<T: IntoIterator<Item = RB>>(iter: T) -> Self {
         let mut res = Self::default();
         for i in iter {
@@ -163,7 +183,7 @@ impl<RB: 'static, const CAP: usize> FromIterator<RB> for ConstGenericRingBuffer<
     }
 }
 
-impl<T: 'static, const CAP: usize> Index<isize> for ConstGenericRingBuffer<T, CAP> {
+impl<T, const CAP: usize> Index<isize> for ConstGenericRingBuffer<T, CAP> {
     type Output = T;
 
     fn index(&self, index: isize) -> &Self::Output {
@@ -171,7 +191,7 @@ impl<T: 'static, const CAP: usize> Index<isize> for ConstGenericRingBuffer<T, CA
     }
 }
 
-impl<T: 'static, const CAP: usize> IndexMut<isize> for ConstGenericRingBuffer<T, CAP> {
+impl<T, const CAP: usize> IndexMut<isize> for ConstGenericRingBuffer<T, CAP> {
     fn index_mut(&mut self, index: isize) -> &mut Self::Output {
         self.get_mut(index).expect("index out of bounds")
     }
